@@ -2,6 +2,7 @@ import os
 import unittest
 from unittest.mock import DEFAULT, patch
 
+import machine_common_sense as mcs
 from machine_common_sense.config_manager import (ChangeMaterialConfig,
                                                  ConfigManager, ForceConfig,
                                                  GoalSchema, MetadataTier,
@@ -74,7 +75,8 @@ class TestConfigManager(unittest.TestCase):
     @mock_env(MCS_CONFIG_FILE_PATH='~/somefolder/env-var-test.ini')
     def test_nonexistent_file_from_env_variable(self):
         '''Missing config files should raise an exception'''
-        self.assertRaises(RuntimeError, ConfigManager)
+        with self.assertRaises(RuntimeError):
+            ConfigManager()
 
     @mock_env(MCS_CONFIG_FILE_PATH=('machine_common_sense/scripts/'
                                     'config_level2_debug.ini'))
@@ -109,11 +111,13 @@ class TestConfigManager(unittest.TestCase):
     def test_init_with_filepath_missing(self):
         '''Provided config file path must exist or an exception occurs'''
         missing_config_file = '~/somefolder/env-var-test.ini'
-        self.assertRaises(RuntimeError, ConfigManager, missing_config_file)
+        with self.assertRaises(RuntimeError):
+            ConfigManager(missing_config_file)
 
     def test_init_with_no_config(self):
         '''No dictionary, no file, and no env should raise an exception'''
-        self.assertRaises(RuntimeError, ConfigManager)
+        with self.assertRaises(RuntimeError):
+            ConfigManager()
 
     def test_init_with_dict(self):
         config_options = {
@@ -283,6 +287,7 @@ class TestSceneConfig(unittest.TestCase):
                 'key': 'value'
             },
             'forces': [{
+                'impulse': True,
                 'relative': True,
                 'stepBegin': 11,
                 'stepEnd': 12,
@@ -303,6 +308,7 @@ class TestSceneConfig(unittest.TestCase):
             'locationParent': 'parent_id',
             'mass': 12.34,
             'materials': ['material_1', 'material_2'],
+            'maxAngularVelocity': 25,
             'moveable': True,
             'moves': [{
                 'stepBegin': 16,
@@ -433,6 +439,7 @@ class TestSceneConfig(unittest.TestCase):
         assert object_1.location_parent is None
         assert object_1.mass is None
         assert object_1.materials is None
+        assert object_1.max_angular_velocity is None
         assert object_1.moveable is None
         assert object_1.moves is None
         assert object_1.null_parent is None
@@ -464,6 +471,7 @@ class TestSceneConfig(unittest.TestCase):
         )]
         assert object_2.debug == {'key': 'value'}
         assert object_2.forces == [ForceConfig(
+            impulse=True,
             relative=True,
             step_begin=11,
             step_end=12,
@@ -478,6 +486,7 @@ class TestSceneConfig(unittest.TestCase):
         assert object_2.location_parent == 'parent_id'
         assert object_2.mass == 12.34
         assert object_2.materials == ['material_1', 'material_2']
+        assert object_2.max_angular_velocity == 25
         assert object_2.moveable is True
         assert object_2.moves == [MoveConfig(
             step_begin=16,
@@ -534,7 +543,7 @@ class TestSceneConfig(unittest.TestCase):
             position=Vector3d(x=0.41, y=0.42, z=0.43)
         )]
         assert object_2.toggle_physics == [SingleStepConfig(step_begin=27)]
-        assert object_2.torques == [MoveConfig(
+        assert object_2.torques == [ForceConfig(
             step_begin=28,
             step_end=29,
             vector=Vector3d(x=0.44, y=0.45, z=0.46)
@@ -587,9 +596,11 @@ class TestSceneConfig(unittest.TestCase):
 
         goal = {
             "action_list": [
-                [("MoveAhead", {"amount": 0.1})],
+                [(mcs.Action.MOVE_AHEAD.value, {"amount": 0.1})],
                 [],
-                [("Pass", {}), ("RotateLeft", {}), ("RotateRight", {})]
+                [(mcs.Action.PASS.value, {}),
+                 (mcs.Action.ROTATE_LEFT.value, {}),
+                 (mcs.Action.ROTATE_RIGHT.value, {})]
             ],
             "category": "test category",
             "description": "test description",
@@ -605,9 +616,10 @@ class TestSceneConfig(unittest.TestCase):
         goal_3 = scene_config.retrieve_goal()
 
         self.assertEqual(goal_3.action_list, [
-            [("MoveAhead", {"amount": 0.1})],
+            [(mcs.Action.MOVE_AHEAD.value, {"amount": 0.1})],
             [],
-            [("Pass", {}), ("RotateLeft", {}), ("RotateRight", {})]
+            [(mcs.Action.PASS.value, {}), (mcs.Action.ROTATE_LEFT.value, {}),
+             (mcs.Action.ROTATE_RIGHT.value, {})]
         ])
         self.assertEqual(goal_3.category, "test category")
         self.assertEqual(goal_3.description, "test description")
