@@ -67,7 +67,10 @@ class TestController(unittest.TestCase):
             rotation={'y': 0.0},
             snapToGrid=False,
             teleportPosition=None,
-            teleportRotation=None
+            teleportRotation=None,
+            clockwise=True,
+            lateral=0,
+            straight=1
         )
 
         for key, value in kwargs.items():
@@ -358,9 +361,8 @@ class TestController(unittest.TestCase):
                 ]}
         }
 
-        self.assertRaises(
-            TypeError,
-            lambda: self.controller.end_scene("1.0", np.float64(0.5), {
+        with self.assertRaises(TypeError):
+            self.controller.end_scene("1.0", np.float64(0.5), {
                 1: {
                     "rating": 1.0,
                     "score": np.float64(.75),
@@ -371,7 +373,6 @@ class TestController(unittest.TestCase):
                         }
                     ]}
             })
-        )
 
     def test_end_scene_twice(self):
         test_payload = self.controller._create_event_payload_kwargs()
@@ -403,9 +404,8 @@ class TestController(unittest.TestCase):
         })
 
         # calling end_scene a second time raises an exception
-        self.assertRaises(
-            RuntimeError,
-            lambda: self.controller.end_scene(
+        with self.assertRaises(RuntimeError):
+            self.controller.end_scene(
                 1.0,
                 0.5,
                 {1: {
@@ -418,7 +418,6 @@ class TestController(unittest.TestCase):
                         }
                     ]}
                  })
-        )
 
     def test_start_scene(self):
         self.controller.set_metadata_tier(
@@ -431,7 +430,7 @@ class TestController(unittest.TestCase):
         )
         self.assertEqual(
             output.action_list,
-            GoalMetadata.ACTION_LIST)
+            GoalMetadata.DEFAULT_ACTIONS)
         self.assertEqual(output.return_status,
                          MOCK_VARIABLES['metadata']['lastActionStatus'])
         self.assertEqual(output.reward, 0)
@@ -451,7 +450,7 @@ class TestController(unittest.TestCase):
         self.assertIsNotNone(output)
         self.assertEqual(
             output.action_list,
-            GoalMetadata.ACTION_LIST)
+            GoalMetadata.DEFAULT_ACTIONS)
         self.assertEqual(output.return_status,
                          MOCK_VARIABLES['metadata']['lastActionStatus'])
         self.assertEqual(output.reward, 0)
@@ -477,7 +476,7 @@ class TestController(unittest.TestCase):
         self.assertIsNotNone(output)
         self.assertEqual(
             output.action_list,
-            GoalMetadata.ACTION_LIST)
+            GoalMetadata.DEFAULT_ACTIONS)
         self.assertEqual(output.return_status,
                          MOCK_VARIABLES['metadata']['lastActionStatus'])
         self.assertEqual(output.reward, -0.005)
@@ -505,11 +504,11 @@ class TestController(unittest.TestCase):
         self.controller.set_metadata_tier(
             MetadataTier.ORACLE.value)
         output = self.controller.start_scene({'name': TEST_FILE_NAME})
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNotNone(output)
         self.assertEqual(
             output.action_list,
-            GoalMetadata.ACTION_LIST)
+            GoalMetadata.DEFAULT_ACTIONS)
         self.assertEqual(output.return_status,
                          MOCK_VARIABLES['metadata']['lastActionStatus'])
         self.assertEqual(output.reward, -0.001)
@@ -525,11 +524,11 @@ class TestController(unittest.TestCase):
         self.assertEqual(len(output.structural_object_list),
                          len(MOCK_VARIABLES['metadata']['structuralObjects']))
 
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNotNone(output)
         self.assertEqual(
             output.action_list,
-            GoalMetadata.ACTION_LIST)
+            GoalMetadata.DEFAULT_ACTIONS)
         self.assertEqual(output.return_status,
                          MOCK_VARIABLES['metadata']['lastActionStatus'])
         self.assertEqual(output.reward, -0.002)
@@ -549,7 +548,7 @@ class TestController(unittest.TestCase):
         self.controller.set_metadata_tier(
             MetadataTier.ORACLE.value)
         output = self.controller.start_scene({'name': TEST_FILE_NAME})
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.controller._publish_event.assert_any_call(
             EventType.ON_BEFORE_STEP,
             ANY
@@ -563,22 +562,22 @@ class TestController(unittest.TestCase):
     def test_step_last_step(self):
         output = self.controller.start_scene({'name': TEST_FILE_NAME})
         self.controller.set_goal(mcs.GoalMetadata(last_step=0))
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNone(output)
 
         self.controller.set_goal(mcs.GoalMetadata(last_step=1))
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNotNone(output)
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNone(output)
 
     def test_retrieve_action_list_at_step(self):
         test_action_list = [
-            ('Pass', {}),
-            ('LookUp', {'amount': 10}),
-            ('MoveAhead', {'amount': 0.1}),
-            ('PickupObject', {'objectId': 'ball'}),
-            ('PickupObject', {'objectId': 'duck'})
+            (mcs.Action.PASS.value, {}),
+            (mcs.Action.LOOK_UP.value, {'amount': 10}),
+            (mcs.Action.MOVE_AHEAD.value, {'amount': 0.1}),
+            (mcs.Action.PICKUP_OBJECT.value, {'objectId': 'ball'}),
+            (mcs.Action.PICKUP_OBJECT.value, {'objectId': 'duck'})
         ]
 
         # With no action list
@@ -587,7 +586,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(),
                 0
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With empty action list
         self.assertEqual(
@@ -595,7 +594,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(action_list=[]),
                 0
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With empty nested action list
         self.assertEqual(
@@ -603,7 +602,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(action_list=[[]]),
                 0
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With test action list
         self.assertEqual(
@@ -619,7 +618,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(action_list=[test_action_list]),
                 1
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With incorrect index
         self.assertEqual(
@@ -627,7 +626,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(action_list=[test_action_list, []]),
                 1
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With incorrect index
         self.assertEqual(
@@ -635,7 +634,7 @@ class TestController(unittest.TestCase):
                 mcs.GoalMetadata(action_list=[[], test_action_list]),
                 0
             ),
-            GoalMetadata.ACTION_LIST
+            GoalMetadata.DEFAULT_ACTIONS
         )
         # With correct index
         self.assertEqual(
@@ -728,7 +727,7 @@ class TestController(unittest.TestCase):
     def test_step_open_action_magnitude(self):
         _ = self.controller.start_scene({'name': TEST_FILE_NAME})
         self.controller.step(
-            'OpenObject',
+            mcs.Action.OPEN_OBJECT.value,
             amount=1,
             objectId='test_id_1',
             receptacleObjectId='test_id_2')
@@ -741,7 +740,7 @@ class TestController(unittest.TestCase):
                 receptacleObjectId='test_id_2'))
 
         self.controller.step(
-            'OpenObject',
+            mcs.Action.OPEN_OBJECT.value,
             amount=0.1,
             objectId='test_id_1',
             receptacleObjectId='test_id_2')
@@ -753,26 +752,22 @@ class TestController(unittest.TestCase):
                 objectId='test_id_1',
                 receptacleObjectId='test_id_2'))
 
-        self.assertRaises(
-            ValueError,
-            lambda: self.controller.step(
-                'OpenObject',
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.OPEN_OBJECT.value,
                 amount=1.5,
                 objectId='test_id_1',
                 receptacleObjectID='test_id_2')
-        )
 
-        self.assertRaises(
-            ValueError,
-            lambda: self.controller.step(
-                'OpenObject',
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.OPEN_OBJECT.value,
                 amount=-1,
                 objectId='test_id_1',
                 receptacleObjectId='test_id_2')
-        )
 
         self.controller.step(
-            'OpenObject',
+            mcs.Action.OPEN_OBJECT.value,
             amount=1,
             objectImageCoordsX=1,
             objectImageCoordsY=2,
@@ -793,120 +788,304 @@ class TestController(unittest.TestCase):
 
     def test_step_validate_action(self):
         _ = self.controller.start_scene({'name': TEST_FILE_NAME})
-        output = self.controller.step('Foobar')
-        self.assertIsNone(output)
+        with self.assertRaises(ValueError):
+            self.controller.step('Foobar')
 
         self.controller.set_goal(mcs.GoalMetadata(action_list=[
-            [('Pass', {})]
+            [(mcs.Action.PASS.value, {})]
         ]))
-        output = self.controller.step('MoveAhead')
-        self.assertIsNone(output)
+        with self.assertRaises(ValueError):
+            self.controller.step(mcs.Action.MOVE_AHEAD.value)
 
         self.controller.set_goal(mcs.GoalMetadata(action_list=[
-            [('MoveAhead', {})],
-            [('MoveBack', {})]]
+            [(mcs.Action.MOVE_AHEAD.value, {})],
+            [(mcs.Action.MOVE_BACK.value, {})]]
         ))
-        output = self.controller.step('MoveAhead')
+        output = self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertIsNotNone(output)
-        output = self.controller.step('MoveAhead')
-        self.assertIsNone(output)
+        with self.assertRaises(ValueError):
+            self.controller.step(mcs.Action.MOVE_AHEAD.value)
 
     def test_step_validate_parameters_move(self):
         _ = self.controller.start_scene({'name': TEST_FILE_NAME})
-        self.controller.step('MoveAhead')
+        self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='MoveAhead',
+                action=mcs.Action.MOVE_AHEAD.value,
                 moveMagnitude=mcs.controller.DEFAULT_MOVE))
 
-        self.controller.step('MoveAhead')
+        self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='MoveAhead',
+                action=mcs.Action.MOVE_AHEAD.value,
                 moveMagnitude=mcs.controller.DEFAULT_MOVE))
 
-        self.controller.step('MoveAhead')
+        self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='MoveAhead',
+                action=mcs.Action.MOVE_AHEAD.value,
                 moveMagnitude=mcs.controller.DEFAULT_MOVE))
 
-        self.controller.step('MoveAhead')
+        self.controller.step(mcs.Action.MOVE_AHEAD.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='MoveAhead',
+                action=mcs.Action.MOVE_AHEAD.value,
                 moveMagnitude=mcs.controller.DEFAULT_MOVE))
 
     def test_step_validate_parameters_rotate(self):
         _ = self.controller.start_scene({'name': TEST_FILE_NAME})
-        self.controller.step('RotateLeft')
+        self.controller.step(mcs.Action.ROTATE_LEFT.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
-            self.create_step_data(action='RotateLeft'))
+            self.create_step_data(action=mcs.Action.ROTATE_LEFT.value))
 
-        self.controller.step('RotateLeft')
+        self.controller.step(mcs.Action.ROTATE_LEFT.value)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='RotateLeft'))
+                action=mcs.Action.ROTATE_LEFT.value))
 
     def test_step_validate_parameters_force_object(self):
         _ = self.controller.start_scene({'name': TEST_FILE_NAME})
         self.controller.step(
-            'PushObject',
+            mcs.Action.PUSH_OBJECT.value,
             force=1,
             objectId='test_id_1')
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='PushObject',
-                horizon=0.0,
-                moveMagnitude=Parameter.MAX_FORCE,
+                action=mcs.Action.PUSH_OBJECT.value,
+                moveMagnitude=1.0,
                 objectId='test_id_1'))
 
         self.controller.step(
-            'PushObject',
+            mcs.Action.PUSH_OBJECT.value,
             force=0.1,
             objectId='test_id_1')
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='PushObject',
-                moveMagnitude=0.1 *
-                Parameter.MAX_FORCE,
+                action=mcs.Action.PUSH_OBJECT.value,
+                moveMagnitude=0.1,
                 objectId='test_id_1'))
 
-        self.assertRaises(
-            ValueError,
-            lambda: self.controller.step(
-                'PushObject',
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.PUSH_OBJECT.value,
                 force=1.5,
                 objectId='test_id_1')
-        )
-        self.assertRaises(
-            ValueError,
-            lambda: self.controller.step(
-                'PushObject',
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.PUSH_OBJECT.value,
                 force=-1,
                 objectId='test_id_1')
-        )
 
         self.controller.step(
-            'PushObject',
+            mcs.Action.PUSH_OBJECT.value,
             force=1,
             objectImageCoordsX=1,
             objectImageCoordsY=2)
         self.assertEqual(
             self.controller.get_last_step_data(),
             self.create_step_data(
-                action='PushObject',
-                moveMagnitude=Parameter.MAX_FORCE,
+                action=mcs.Action.PUSH_OBJECT.value,
+                moveMagnitude=1.0,
                 objectImageCoords={'x': 1, 'y': 397}))
 
+        self.controller.step(
+            mcs.Action.PUSH_OBJECT.value,
+            objectId='test_id_1')
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.PUSH_OBJECT.value,
+                moveMagnitude=0.5,
+                objectId='test_id_1'))
+
+    def test_step_validate_parameters_torque_object(self):
+        _ = self.controller.start_scene({'name': TEST_FILE_NAME})
+        self.controller.step(
+            mcs.Action.TORQUE_OBJECT.value,
+            force=1,
+            objectImageCoordsX=1,
+            objectImageCoordsY=2)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.TORQUE_OBJECT.value,
+                moveMagnitude=1.0,
+                objectImageCoords={'x': 1, 'y': 397}))
+
+        self.controller.step(
+            mcs.Action.TORQUE_OBJECT.value,
+            force=-1,
+            objectImageCoordsX=1,
+            objectImageCoordsY=2)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.TORQUE_OBJECT.value,
+                moveMagnitude=-1.0,
+                objectImageCoords={'x': 1, 'y': 397}))
+
+        self.controller.step(
+            mcs.Action.TORQUE_OBJECT.value,
+            objectImageCoordsX=1,
+            objectImageCoordsY=2)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.TORQUE_OBJECT.value,
+                moveMagnitude=0.5,
+                objectImageCoords={'x': 1, 'y': 397}))
+
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.TORQUE_OBJECT.value,
+                force=1.1,
+                objectId='test_id_1')
+
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.TORQUE_OBJECT.value,
+                force=-1.1,
+                objectId='test_id_1')
+
+    def test_step_validate_parameters_rotate_object(self):
+        _ = self.controller.start_scene({'name': TEST_FILE_NAME})
+        self.controller.step(
+            mcs.Action.ROTATE_OBJECT.value,
+            objectId='test_id_1')
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.ROTATE_OBJECT.value,
+                objectId='test_id_1',
+                clockwise=True))
+
+        self.controller.step(
+            mcs.Action.ROTATE_OBJECT.value,
+            objectId='test_id_1',
+            clockwise='false')
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.ROTATE_OBJECT.value,
+                objectId='test_id_1',
+                clockwise=False))
+
+        self.controller.step(
+            mcs.Action.ROTATE_OBJECT.value,
+            objectId='test_id_1',
+            clockwise='False')
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.ROTATE_OBJECT.value,
+                objectId='test_id_1',
+                clockwise=False))
+
+        self.controller.step(
+            mcs.Action.ROTATE_OBJECT.value,
+            objectId='test_id_1',
+            clockwise=True)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.ROTATE_OBJECT.value,
+                objectId='test_id_1',
+                clockwise=True))
+
+        self.controller.step(
+            mcs.Action.ROTATE_OBJECT.value,
+            objectId='test_id_1',
+            clockwise=False)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action=mcs.Action.ROTATE_OBJECT.value,
+                objectId='test_id_1',
+                clockwise=False))
+
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.ROTATE_OBJECT.value,
+                clockwise=1.0,
+                objectId='test_id_1')
+
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                mcs.Action.ROTATE_OBJECT.value,
+                clockwise='string',
+                objectId='test_id_1')
+
+    def test_step_validate_parameters_move_object(self):
+        _ = self.controller.start_scene({'name': TEST_FILE_NAME})
+        self.controller.step('MoveObject', objectId='test_id_1')
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action='MoveObject',
+                objectId='test_id_1',
+                lateral=0,
+                straight=1))
+        self.controller.step(
+            'MoveObject',
+            objectId='test_id_1',
+            lateral=1)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action='MoveObject',
+                objectId='test_id_1',
+                lateral=1,
+                straight=0))
+        self.controller.step(
+            'MoveObject',
+            objectId='test_id_1',
+            lateral=1,
+            straight=-1)
+        self.assertEqual(
+            self.controller.get_last_step_data(),
+            self.create_step_data(
+                action='MoveObject',
+                objectId='test_id_1',
+                lateral=1,
+                straight=-1))
+
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                'MoveObject',
+                lateral='string',
+                objectId='test_id_1')
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                'MoveObject',
+                straight='string',
+                objectId='test_id_1')
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                'MoveObject',
+                lateral=1,
+                straight=0.1,
+                objectId='test_id_1')
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                'MoveObject',
+                lateral=0.1,
+                straight=1,
+                objectId='test_id_1')
+        with self.assertRaises(ValueError):
+            self.controller.step(
+                'MoveObject',
+                lateral=True,
+                straight=False,
+                objectId='test_id_1')
+
 
 if __name__ == '__main__':
     unittest.main()
